@@ -1,8 +1,8 @@
-__version__         = '0.3.2'
+__version__         = '0.4.0'
 __author__          = "Dave Martorana (http://davemartorana.com), Richard Cooper (http://frozenskys.com), Bill Jones (oraclebill), Dmitry Golomidov (deeGraYve)"
 __date__            = '2010-April-14'
 __url__             = 'http://postmarkapp.com'
-__copyright__       = "(C) 2009-2012 David Martorana, Wildbit LLC, Python Software Foundation."
+__copyright__       = "(C) 2009-2014 David Martorana, Wildbit LLC, Python Software Foundation."
 __contributors__    = "Dave Martorana (themartorana), Bill Jones (oraclebill), Richard Cooper (frozenskys), Miguel Araujo (maraujop), Patrick Lauber (digi604), Brian McFadden (brimcfadden), Joel Ryan (joelryan2k), Ben Hodgson (benhodgson), Dmitry Golomidov (deeGraYve)"
 
 #
@@ -10,13 +10,20 @@ __contributors__    = "Dave Martorana (themartorana), Bill Jones (oraclebill), R
 
 try:
     from email.mime.base import MIMEBase
-except ImportError, e:
+except ImportError as e:
     from email import MIMEBase
-    
+    from httplib import HTTPConnection
+    from urllib import urlencode
 import sys
-import urllib
-import urllib2
-import httplib
+if sys.version_info[0] < 3:
+    from urllib2 import Request, urlopen, HTTPError, URLError
+else:
+    from urllib.request import Request, urlopen
+    from urllib.error import HTTPError, URLError
+    from http.client import HTTPConnection
+    from urllib.parse import urlencode
+
+import collections
 
 try:
     import simplejson as json
@@ -31,17 +38,17 @@ except ImportError:
             raise Exception('Cannot use python-postmark library without Python 2.6 or greater, or Python 2.4 or 2.5 and the "simplejson" library')
 
 class PMJSONEncoder(json.JSONEncoder):
-	def default(self, o):
-		try:
-			if hasattr(o, '_proxy____str_cast') and callable(o._proxy____str_cast):
-				return o._proxy____str_cast()
-			elif hasattr(o, '_proxy____unicode_cast'):
-				return unicode(o)
-		except:
-			pass
-			
-		return super(PMJSONEncoder, self).default(o)
-	
+    def default(self, o):
+        try:
+            if hasattr(o, '_proxy____str_cast') and isinstance(o._proxy____str_cast, collections.Callable):
+                return o._proxy____str_cast()
+            elif hasattr(o, '_proxy____unicode_cast'):
+                return str(o)
+        except:
+            pass
+
+        return super(PMJSONEncoder, self).default(o)
+
 #
 #
 __POSTMARK_URL__ = 'https://api.postmarkapp.com/'
@@ -87,33 +94,33 @@ class PMMail(object):
         self.__custom_headers = {}
         self.__attachments = []
         #self.__multipart = False
-        
+
         acceptable_keys = (
-            'api_key', 
-            'sender', 
+            'api_key',
+            'sender',
             'reply_to',
             'to', 'recipient', # 'recipient' is legacy
             'cc',
-            'bcc', 
-            'subject', 
+            'bcc',
+            'subject',
             'tag',
-            'html_body', 
-            'text_body', 
+            'html_body',
+            'text_body',
             'custom_headers',
             'attachments',
             #'multipart'
         )
-        
+
         for key in kwargs:
             if key in acceptable_keys:
                 if key == 'recipient':
                     setattr(self, '_PMMail__to', kwargs[key])
                 else:
                     setattr(self, '_PMMail__%s' % key, kwargs[key])
-                
+
         # Set up the user-agent
         self.__user_agent = 'Python/%s (python-postmark library version %s)' % ('_'.join([str(var) for var in sys.version_info]), __version__)
-        
+
         # Try to pull in the API key from Django
         try:
             from django import VERSION
@@ -125,13 +132,13 @@ class PMMail(object):
                 self.__sender = django_settings.POSTMARK_SENDER
         except ImportError:
             pass
-        
+
     #
     # Properties
-    
+
     def _set_custom_headers(self, value):
         '''
-        A special set function to ensure 
+        A special set function to ensure
         we're setting with a dictionary
         '''
         if value == None:
@@ -140,7 +147,7 @@ class PMMail(object):
             setattr(self, '_PMMail__custom_headers', value)
         else:
             raise TypeError('Custom headers must be a dictionary of key-value pairs')
-        
+
     def _set_attachments(self, value):
         '''
         A special set function to ensure
@@ -152,11 +159,11 @@ class PMMail(object):
             setattr(self, '_PMMail__attachments', value)
         else:
             raise TypeError('Attachments must be a list')
-    
+
     api_key = property(
         lambda self: self.__api_key,
         lambda self, value: setattr(self, '_PMMail__api_key', value),
-        lambda self: setattr(self, '_PMMail__api_key', None), 
+        lambda self: setattr(self, '_PMMail__api_key', None),
         '''
         The API Key for your rack server on Postmark
         '''
@@ -167,25 +174,25 @@ class PMMail(object):
         lambda self: self.__sender,
         lambda self, value: setattr(self, '_PMMail__sender', value),
         lambda self: setattr(self, '_PMMail__sender', None),
-        '''        
-        The sender, in either "name@email.com" or "First Last <name@email.com>" formats.  
+        '''
+        The sender, in either "name@email.com" or "First Last <name@email.com>" formats.
         The address should match one of your Sender Signatures in Postmark.
         Specifying the address in the second fashion will allow you to replace
         the name of the sender as it appears in the recipient's email client.
         '''
     )
-        
+
     reply_to = property(
         lambda self: self.__reply_to,
         lambda self, value: setattr(self, '_PMMail__reply_to', value),
         lambda self: setattr(self, '_PMMail__reply_to', None),
         '''
-        A reply-to address, in either "name@email.com" or "First Last <name@email.com>" 
+        A reply-to address, in either "name@email.com" or "First Last <name@email.com>"
         format. The reply-to address does not have to be one of your Sender Signatures in Postmark.
         This allows a different reply-to address than sender address.
         '''
     )
-     
+
     to = property(
         lambda self: self.__to,
         lambda self, value: setattr(self, '_PMMail__to', value),
@@ -194,7 +201,7 @@ class PMMail(object):
         The recipients, in either "name@email.com" or "First Last <name@email.com>" formats
         '''
     )
-    
+
     cc = property(
         lambda self: self.__cc,
         lambda self, value: setattr(self, '_PMMail__cc', value),
@@ -203,7 +210,7 @@ class PMMail(object):
         The cc recipients, in either "name@email.com" or "First Last <name@email.com>" formats
         '''
     )
-    
+
     bcc = property(
         lambda self: self.__bcc,
         lambda self, value: setattr(self, '_PMMail__bcc', value),
@@ -212,7 +219,7 @@ class PMMail(object):
         The bcc recipients, in either "name@email.com" or "First Last <name@email.com>" formats
         '''
     )
-    
+
     subject = property(
         lambda self: self.__subject,
         lambda self, value: setattr(self, '_PMMail__subject', value),
@@ -227,12 +234,12 @@ class PMMail(object):
         lambda self, value: setattr(self, '_PMMail__tag', value),
         lambda self: setattr(self, '_PMMail__tag', None),
         '''
-        You can categorize outgoing email using the optional Tag property. 
-        If you use different tags for the different types of emails your application generates, 
+        You can categorize outgoing email using the optional Tag property.
+        If you use different tags for the different types of emails your application generates,
         you will be able to get detailed statistics for them through the Postmark user interface.
         '''
     )
-    
+
     html_body = property(
         lambda self: self.__html_body,
         lambda self, value: setattr(self, '_PMMail__html_body', value),
@@ -241,7 +248,7 @@ class PMMail(object):
         The email message body, in html format
         '''
     )
-    
+
     text_body = property(
         lambda self: self.__text_body,
         lambda self, value: setattr(self, '_PMMail__text_body', value),
@@ -250,13 +257,13 @@ class PMMail(object):
         The email message body, in text format
         '''
     )
-    
+
     custom_headers = property(
         lambda self: self.__custom_headers,
-        _set_custom_headers, 
+        _set_custom_headers,
         lambda self: setattr(self, '_PMMail__custom_headers', {}),
         '''
-        Custom headers in a standard dictionary. 
+        Custom headers in a standard dictionary.
         NOTE: To change the reply to address, use the .reply_to
         property instead of a custom header.
         '''
@@ -270,20 +277,20 @@ class PMMail(object):
         Attachments, Base64 encoded, in a list.
         '''
         )
-    
+
 #     multipart = property(
 #         lambda self: self.__multipart,
 #         lambda self, value: setattr(self, '_PMMail__multipart', value),
 #         'The API Key for one of your servers on Postmark'
 #     )
-        
+
 
     #####################
     #
-    # LEGACY SUPPORT 
+    # LEGACY SUPPORT
     #
     #####################
-    
+
     recipient = property(
         lambda self: self.__to,
         lambda self, value: setattr(self, '_PMMail__to', value),
@@ -295,11 +302,11 @@ class PMMail(object):
 
     #####################
     #
-    # END LEGACY SUPPORT 
+    # END LEGACY SUPPORT
     #
     #####################
 
-    
+
     def _check_values(self):
         '''
         Make sure all values are of the appropriate
@@ -372,19 +379,19 @@ class PMMail(object):
 
     def send(self, test=None):
         '''
-        Send the email through the Postmark system.  
+        Send the email through the Postmark system.
         Pass test=True to just print out the resulting
         JSON message being sent to Postmark
         '''
         self._check_values()
-        
+
         # Set up message dictionary
         json_message = self.to_json_message()
-            
+
 #         if (self.__html_body and not self.__text_body) and self.__multipart:
 #             # TODO: Set up regex to strip html
 #             pass
-        
+
         # If test is not specified, attempt to read the Django setting
         if test is None:
             try:
@@ -392,16 +399,16 @@ class PMMail(object):
                 test = getattr(django_settings, "POSTMARK_TEST_MODE", None)
             except ImportError:
                 pass
-        
+
         # If this is a test, just print the message
         if test:
-            print 'JSON message is:\n%s' % json.dumps(json_message, cls=PMJSONEncoder)
+            print('JSON message is:\n%s' % json.dumps(json_message, cls=PMJSONEncoder))
             return
-            
+
         # Set up the url Request
-        req = urllib2.Request(
+        req = Request(
             __POSTMARK_URL__ + 'email',
-            json.dumps(json_message, cls=PMJSONEncoder),
+            json.dumps(json_message, cls=PMJSONEncoder).encode('utf8'),
             {
                 'Accept': 'application/json',
                 'Content-Type': 'application/json',
@@ -409,17 +416,17 @@ class PMMail(object):
                 'User-agent': self.__user_agent
             }
         )
-        
+
         # Attempt send
         try:
             #print 'sending request to postmark: %s' % json_message
-            result = urllib2.urlopen(req)
+            result = urlopen(req)
             result.close()
             if result.code == 200:
                 return True
             else:
                 raise PMMailSendException('Return code %d: %s' % (result.code, result.msg))
-        except urllib2.HTTPError, err:
+        except HTTPError as err:
             if err.code == 401:
                 raise PMMailUnauthorizedException('Sending Unauthorized - incorrect API key.', err)
             elif err.code == 422:
@@ -432,7 +439,7 @@ class PMMail(object):
                 raise PMMailUnprocessableEntityException('Unprocessable Entity: %s' % desc)
             elif err.code == 500:
                 raise PMMailServerErrorException('Internal server error at Postmark. Admins have been alerted.', err)
-        except urllib2.URLError, err:
+        except URLError as err:
             if hasattr(err, 'reason'):
                 raise PMMailURLException('URLError: Failed to reach the server: %s (See "inner_exception" for details)' % err.reason, err)
             elif hasattr(err, 'code'):
@@ -474,11 +481,11 @@ class PMBatchMail(object):
                 self.__api_key = django_settings.POSTMARK_API_KEY
         except ImportError:
             pass
-    
+
     api_key = property(
         lambda self: self.__api_key,
         lambda self, value: setattr(self, '_PMBatchMail__api_key', value),
-        lambda self: setattr(self, '_PMBatchMail__api_key', None), 
+        lambda self: setattr(self, '_PMBatchMail__api_key', None),
         '''
         The API Key for your rack server on Postmark
         '''
@@ -487,7 +494,7 @@ class PMBatchMail(object):
     messages = property(
         lambda self: self.__messages,
         lambda self, value: setattr(self, '_PMBatchMail__messages', value),
-        lambda self: setattr(self, '_PMBatchMail__messages', None), 
+        lambda self: setattr(self, '_PMBatchMail__messages', None),
         '''
         Messages to send
         '''
@@ -532,9 +539,9 @@ class PMBatchMail(object):
             for message in messages:
                 json_message.append(message.to_json_message())
 
-            req = urllib2.Request(
+            req = Request(
                 __POSTMARK_URL__ + 'email/batch',
-                json.dumps(json_message, cls=PMJSONEncoder),
+                json.dumps(json_message, cls=PMJSONEncoder).encode('utf8'),
                 {
                     'Accept': 'application/json',
                     'Content-Type': 'application/json',
@@ -545,18 +552,18 @@ class PMBatchMail(object):
 
             # If this is a test, just print the message
             if test:
-                print 'JSON message is:\n%s' % json.dumps(json_message, cls=PMJSONEncoder)
+                print('JSON message is:\n%s' % json.dumps(json_message, cls=PMJSONEncoder))
                 continue
 
             # Attempt send
             try:
-                result = urllib2.urlopen(req)
+                result = urlopen(req)
                 result.close()
                 if result.code == 200:
                     pass
                 else:
                     raise PMMailSendException('Return code %d: %s' % (result.code, result.msg))
-            except urllib2.HTTPError, err:
+            except HTTPError as err:
                 if err.code == 401:
                     raise PMMailUnauthorizedException('Sending Unauthorized - incorrect API key.', err)
                 elif err.code == 422:
@@ -569,7 +576,7 @@ class PMBatchMail(object):
                     raise PMMailUnprocessableEntityException('Unprocessable Entity: %s' % desc)
                 elif err.code == 500:
                     raise PMMailServerErrorException('Internal server error at Postmark. Admins have been alerted.', err)
-            except urllib2.URLError, err:
+            except URLError as err:
                 if hasattr(err, 'reason'):
                     raise PMMailURLException('URLError: Failed to reach the server: %s (See "inner_exception" for details)' % err.reason, err)
                 elif hasattr(err, 'code'):
@@ -591,19 +598,19 @@ class PMBounceManager(object):
         '''
         # initialize properties
         self.__api_key = None
-        
-        
+
+
         acceptable_keys = (
-            'api_key', 
+            'api_key',
         )
 
         for key in kwargs:
             if key in acceptable_keys:
                 setattr(self, '_PMBounceManager__%s' % key, kwargs[key])
-                
+
         # Set up the user-agent
         self.__user_agent = 'Python/%s (python-postmark library version %s)' % ('_'.join([str(var) for var in sys.version_info]), __version__)
-        
+
         # Try to pull in the API key from Django
         try:
             from django import VERSION
@@ -613,31 +620,31 @@ class PMBounceManager(object):
             self.__user_agent = '%s (Django %s)' % (self.__user_agent, '_'.join([str(var) for var in VERSION]))
         except ImportError:
             pass
-            
+
     def _check_values(self):
         '''
         Make sure all values are of the appropriate
         type and are not missing.
         '''
         if not self.__api_key:
-            raise PMMailMissingValueException('Cannot check bounces without a Postmark Server API Key')      
+            raise PMMailMissingValueException('Cannot check bounces without a Postmark Server API Key')
 
     api_key = property(
         lambda self: self.__api_key,
         lambda self, value: setattr(self, '_PMMail__api_key', value),
-        lambda self: setattr(self, '_PMMail__api_key', None), 
+        lambda self: setattr(self, '_PMMail__api_key', None),
         '''
         The API Key for your rack server on Postmark
         '''
-    )       
-    
+    )
+
     def delivery_stats(self):
         '''
         Returns a summary of inactive emails and bounces by type.
         '''
         self._check_values()
-        
-        req = urllib2.Request(
+
+        req = Request(
             __POSTMARK_URL__ + 'deliverystats',
             None,
             {
@@ -647,37 +654,37 @@ class PMBounceManager(object):
                 'User-agent': self.__user_agent
             }
         )
-        
+
         # Attempt send
         try:
             #print 'sending request to postmark:'
-            result = urllib2.urlopen(req)
+            result = urlopen(req)
             if result.code == 200:
                 return json.loads(result.read())
                 result.close()
             else:
-            	result.close()
+                result.close()
                 raise PMMailSendException('Return code %d: %s' % (result.code, result.msg))
-            
-        except urllib2.HTTPError, err:
+
+        except HTTPError as err:
             return err
-                            
-                            
+
+
     def get_all(self, inactive='', email_filter='', tag='', count=25, offset=0):
-    	'''
-        Fetches a portion of bounces according to the specified input criteria. The count and offset 
-        parameters are mandatory. You should never retrieve all bounces as that could be excessively 
-        slow for your application. To know how many bounces you have, you need to request a portion 
-        first, usually the first page, and the service will return the count in the TotalCount property 
+        '''
+        Fetches a portion of bounces according to the specified input criteria. The count and offset
+        parameters are mandatory. You should never retrieve all bounces as that could be excessively
+        slow for your application. To know how many bounces you have, you need to request a portion
+        first, usually the first page, and the service will return the count in the TotalCount property
         of the response.
         '''
-        
+
         self._check_values()
-        
-        params = '?inactive=' + inactive + '&emailFilter=' + email_filter +'&tag=' + tag 
+
+        params = '?inactive=' + inactive + '&emailFilter=' + email_filter +'&tag=' + tag
         params += '&count=' + str(count) + '&offset=' + str(offset)
-        
-        req = urllib2.Request(  	
+
+        req = Request(
             __POSTMARK_URL__ + 'bounces' + params,
             None,
             {
@@ -687,30 +694,30 @@ class PMBounceManager(object):
                 'User-agent': self.__user_agent
             }
         )
-        
+
         # Attempt send
         try:
             #print 'sending request to postmark:'
-            result = urllib2.urlopen(req)
+            result = urlopen(req)
             if result.code == 200:
                 return json.loads(result.read())
                 result.close()
             else:
-            	result.close()
+                result.close()
                 raise PMMailSendException('Return code %d: %s' % (result.code, result.msg))
-            
-        except urllib2.HTTPError, err:
+
+        except HTTPError as err:
             return err
-        
-        
+
+
     def get_single(self, bounce_id):
-    	'''
-    	Get details about a single bounce. Note that the bounce ID is a numeric value that you 
-    	typically obtain after a getting a list of bounces.
-    	'''
+        '''
+        Get details about a single bounce. Note that the bounce ID is a numeric value that you
+        typically obtain after a getting a list of bounces.
+        '''
         self._check_values()
-        
-        req = urllib2.Request(
+
+        req = Request(
             __POSTMARK_URL__ + '/bounces/' + str(bounce_id),
             None,
             {
@@ -720,34 +727,34 @@ class PMBounceManager(object):
                 'User-agent': self.__user_agent
             }
         )
-        
+
         # Attempt send
         try:
             #print 'sending request to postmark:'
-            result = urllib2.urlopen(req)
+            result = urlopen(req)
             if result.code == 200:
                 return json.loads(result.read())
                 result.close()
             else:
-            	result.close()
+                result.close()
                 raise PMMailSendException('Return code %d: %s' % (result.code, result.msg))
-            
-        except urllib2.HTTPError, err:
+
+        except HTTPError as err:
             return err
-    
-    
+
+
     def get_dump(self, bounce_id):
         '''
-    	Returns the raw source of the bounce Postmark accepted. If Postmark does not have a dump for 
+        Returns the raw source of the bounce Postmark accepted. If Postmark does not have a dump for
         that bounce, it will return an empty string.
-    	'''
+        '''
         self._check_values()
 
         req_url = __POSTMARK_URL__ + '/bounces/' + str(bounce_id) + '/dump'
         #print req_url
-        
-        req = urllib2.Request(
-	    req_url,
+
+        req = Request(
+        req_url,
             None,
             {
                 'Accept': 'application/json',
@@ -756,28 +763,28 @@ class PMBounceManager(object):
                 'User-agent': self.__user_agent
             }
         )
-        
+
         # Attempt send
         try:
-            print 'sending request to postmark:'
-            result = urllib2.urlopen(req)
+            print('sending request to postmark:')
+            result = urlopen(req)
             if result.code == 200:
                 return json.loads(result.read())
                 result.close()
             else:
-            	result.close()
+                result.close()
                 raise PMMailSendException('Return code %d: %s' % (result.code, result.msg))
-            
-        except urllib2.HTTPError, err:
+
+        except HTTPError as err:
             return err
-        
+
     def get_tags(self):
         '''
-    	Returns a list of tags used for the current server.
-    	'''
+        Returns a list of tags used for the current server.
+        '''
         self._check_values()
-        
-        req = urllib2.Request(
+
+        req = Request(
             __POSTMARK_URL__ + 'bounces/tags',
             None,
             {
@@ -787,32 +794,32 @@ class PMBounceManager(object):
                 'User-agent': self.__user_agent
             }
         )
-        
+
         # Attempt send
         try:
             #print 'sending request to postmark:'
-            result = urllib2.urlopen(req)
+            result = urlopen(req)
             if result.code == 200:
                 return json.loads(result.read())
                 result.close()
             else:
-            	result.close()
+                result.close()
                 raise PMMailSendException('Return code %d: %s' % (result.code, result.msg))
-            
-        except urllib2.HTTPError, err:
+
+        except HTTPError as err:
             return err
-        
+
     def activate(self, bounce_id):
         '''
-    	Activates a deactivated bounce.
-    	'''
+        Activates a deactivated bounce.
+        '''
         self._check_values()
         req_url = '/bounces/' + str(bounce_id) + '/activate'
         #print req_url
-        h1 = httplib.HTTPConnection('api.postmarkapp.com')
-        dta = urllib.urlencode({"data":"blank"})
+        h1 = HTTPConnection('api.postmarkapp.com')
+        dta = urlencode({"data":"blank"}).encode('utf8')
         req = h1.request('PUT',
-	    req_url,
+        req_url,
             dta,
             {
                 'Accept': 'application/json',
@@ -821,9 +828,9 @@ class PMBounceManager(object):
                 'User-agent': self.__user_agent
             }
         )
-	r=h1.getresponse()
-	return json.loads(r.read())
-        
+        r=h1.getresponse()
+        return json.loads(r.read())
+
 
 #
 # Exceptions
@@ -857,7 +864,7 @@ class PMMailUnprocessableEntityException(PMMailSendException):
     details for further information
     '''
     pass
-    
+
 class PMMailServerErrorException(PMMailSendException):
     '''
     500: Internal error - this is on the Postmark server side.  Errors are
